
#ifndef _NATOOLS_H_
#define _NATOOLS_H_ 1

#include <EXTERN.h>
#include <perl.h>

// Some types................
#define nat_boolean_t U8
#define nat_uchar_t   U8
#define nat_int_t     IV
#define nat_uint32_t  U32

#include <NATools/config.h>
#include <NATools/words.h>
#include <NATools/corpus.h>

#endif /* _NATOOLS_H_ */

